#pragma once

#ifndef __CONTROLRT_H__
#define __CONTROLRT_H__

#include <string>
#include "rtcpal.h"
#include "etwrtcommon.h"
#include "shmmanager.h"

namespace etw {


class ControllerRT {
public:
    /// May throw boost::interprocess::interprocess_exception
    ControllerRT(const std::string & providerGuid) :
        m_providerGuid(providerGuid),
        m_shmMgr(providerGuid)
    {
    }

    /// May throw boost::interprocess::interprocess_exception
    ControllerRT(const GUID & providerGuid) :
        m_providerGuid(StringFromGUID(&providerGuid)),
        m_shmMgr(m_providerGuid)
    {
    }

    ~ControllerRT() {}

    ULONG StartTrace() const {
        std::string etlfile = ETWRT::GetEtlFullPath(m_providerGuid);
        return ETWRT::CreateFIFO(etlfile);
    }

    ULONG EnableTrace() {
        if (!m_shmMgr.EnableAll(ETWRT::SendSignal)) {
            return ERROR_GEN_FAILURE;
        }
        return ERROR_SUCCESS;
    }

    ULONG DisableTrace() {
        if (!m_shmMgr.DisableAll(ETWRT::SendSignal)) {
            return ERROR_GEN_FAILURE;
        }
        return ERROR_SUCCESS;
    }

private:
    std::string m_providerGuid;
    ShmManager m_shmMgr;
};

} // namespace etw

#endif // __CONTROLRT_H__
