#pragma once
#ifndef __RTCPAL_STD_H__
#define __RTCPAL_STD_H__

#if defined (__GNUC__) && !defined (__clang__)
// for gcc 4.9.0, GCC_VERSION will be 40900
#define GCC_VERSION (__GNUC__ * 10000  + __GNUC_MINOR__ * 100 + __GNUC_PATCHLEVEL__)
#endif // __GNUC__


// Some C++ compiler doesn't support <codecvt>.
// for example, GCC with version < 4.9 doesn't support it.
// not sure which GCC starts to support it.

#if defined(__clang__) || (GCC_VERSION > 50000)
#define RTCPAL_CODECVT_SUPPORT 1
#endif //__clang__ || GCC_VERSION

// Some C++ compiler doesn't support std::make_unique<T>.
// for example, GCC with version < 4.9 doesn't support it.
#if defined (__GNUC__) && !defined (__clang__)

#if GCC_VERSION < 40900

#include <memory>
namespace std {

    /// <summary>
    /// Simplistic implementation of make_unique. Copied from CPPREST API.
    /// </summary>
    template <typename _Type>
    std::unique_ptr<_Type> make_unique() {
        return std::unique_ptr<_Type>(new _Type());
    }

    template <typename _Type, typename _Arg1>
    std::unique_ptr<_Type> make_unique(_Arg1&& arg1) {
        return std::unique_ptr<_Type>(new _Type(std::forward<_Arg1>(arg1)));
    }

    template <typename _Type, typename _Arg1, typename _Arg2>
    std::unique_ptr<_Type> make_unique(_Arg1&& arg1, _Arg2&& arg2) {
        return std::unique_ptr<_Type>(new _Type(std::forward<_Arg1>(arg1), std::forward<_Arg2>(arg2)));
    }

    template <typename _Type, typename _Arg1, typename _Arg2, typename _Arg3>
    std::unique_ptr<_Type> make_unique(_Arg1&& arg1, _Arg2&& arg2, _Arg3&& arg3) {
        return std::unique_ptr<_Type>(new _Type(std::forward<_Arg1>(arg1), std::forward<_Arg2>(arg2), std::forward<_Arg3>(arg3)));
    }
}
#endif // GCC_VERSION
#endif // __GNUC__

#endif // __RTCPAL_STD_H__